@@ -11,8 +11,8 @@
 
 var foxyproxy = {
   checkboxType : Components.interfaces.nsITreeColumn.TYPE_CHECKBOX,
-  fp : Components.classes["@leahscape.org/foxyproxy/service;1"]
-         .getService(Components.interfaces.nsISupports).wrappedJSObject,
+  fp : Components.classes["@leahscape.org/foxyproxy/service;1"].getService().wrappedJSObject,
+  fpc : Components.classes["@leahscape.org/foxyproxy/common;1"].getService().wrappedJSObject,
   statusIcon : null,
   contextMenuIcon : null,
   toolbarIcon : null,
@@ -26,7 +26,75 @@ var foxyproxy = {
       .alert(null, this.fp.getMessage("foxyproxy"), str);
   },
 
-	observe: function(subj, topic, str) {
+  // thanks, mzz
+  updateCheck : {
+    timer : null,
+    first : false,
+    usingObserver : true,
+    check : function() {
+      const CC = Components.classes, CI = Components.interfaces;
+      var vc = CC["@mozilla.org/xpcom/version-comparator;1"].getService(CI.nsIVersionComparator),
+        lastVer, curVer = CC["@leahscape.org/foxyproxy/common;1"].getService().wrappedJSObject.getVersion();
+      try {
+        lastVer = foxyproxy.fp.getPrefsService("extensions.foxyproxy.").getCharPref("last-version");
+        if (vc.compare(curVer, lastVer) > 0)
+          p(this);
+      }
+      catch(e) {
+        this.first = true;
+        p(this);
+      }
+      function p(o) {
+        var appInfo = CC["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULAppInfo);
+        if(vc.compare(appInfo.version, "3.0") < 0) {
+          // sessionstore-windows-restored notification not supported; just do it now
+          o.usingObserver = false;
+          o.installTimer();
+        }
+        else
+          CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService)
+             .addObserver(o, "sessionstore-windows-restored", false);
+      }
+    },
+
+    notify : function() {
+      if (this.usingObserver)
+        Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService)
+          .removeObserver(this, "sessionstore-windows-restored");
+
+      // Probably not necessary, but does not hurt
+      this.timer = null;
+      var fpc = Components.classes["@leahscape.org/foxyproxy/common;1"].getService().wrappedJSObject;
+      if (this.first) {
+        x("http://foxyproxy.mozdev.org/help.html");
+        foxyproxy.torWizard(true);
+      }
+      else {
+         x("http://foxyproxy.mozdev.org/releasenotes.html");
+      }
+      function x(url) {
+        fpc.openAndReuseOneTabPerURL(url);
+        // Do this last so we try again next time if we failed to display now
+        foxyproxy.fp.getPrefsService("extensions.foxyproxy.").setCharPref("last-version", fpc.getVersion());
+      }
+    },
+
+   	observe : function(s, topic) {
+      if (topic == "sessionstore-windows-restored") {
+        // If we show the tab now, the tab isn't guaranteed to be topmost
+        // (in Firefox 3.0b5). So use a timer.
+        this.installTimer();
+      }
+    },
+
+    installTimer : function() {
+      this.timer = Components.classes["@mozilla.org/timer;1"].createInstance(Components.interfaces.nsITimer);
+      this.timer.initWithCallback(this, 1000, Components.interfaces.nsITimer.TYPE_ONE_SHOT);
+    }
+  },
+
+
+	observe : function(subj, topic, str) {
     var e;
 		try {
 			e = subj.QueryInterface(Components.interfaces.nsISupportsPRBool).data;
@@ -65,7 +133,6 @@ var foxyproxy = {
   },
 
   onLoad : function() {
-    this.fp.init();
   	this.statusIcon = document.getElementById("foxyproxy-status-icon");
   	this.contextMenuIcon = document.getElementById("foxyproxy-context-menu-1");
   	this.toolbarIcon = document.getElementById("foxyproxy-button-1");
@@ -78,10 +145,11 @@ var foxyproxy = {
     this.toggleContextMenu(this.fp.contextMenu);
     this.checkPageLoad();
     this.toggleStatusBarIcon(this.fp.statusbar.iconEnabled);
-    this.toggleStatusBarText(this.fp.statusbar.textEnabled);    
+    this.toggleStatusBarText(this.fp.statusbar.textEnabled);
     this.toggleStatusBarWidth(this.fp.statusbar.width);
 		this.setMode(this.fp.mode);
-    this._firstRunCheck();
+    this.fp.notifier.emptyQueue();
+    this.updateCheck.check();
   },
 
   toggleToolsMenu : function(e) {
@@ -92,26 +160,6 @@ var foxyproxy = {
  	 	this.contextMenuIcon.hidden = !e;
   },
 
-  onPageLoad : function(evt) {
-	  var doc = evt.originalTarget; // doc is document that triggered "onload" event
-	  (doc && doc.location && foxyproxy.fp.autoadd.perform(doc.location, doc.documentElement.innerHTML) && foxyproxy.fp.writeSettings());
-  },
-
-
-  // The first time FP runs, "firstrun" does not exist (i.e., null || false). Subsequent times, "firstrun" == true.
-  // In other words, this pref is improperly named for its purpose. Better name is "notfirstrun".
-  _firstRunCheck : function() {
-    var f = false;
-    try {
-      f = this.fp.getPrefsService("extensions.foxyproxy.").getBoolPref("firstrun");
-    }
-    catch(e) {}
-    if (!f) {
-      this.torWizard(true);
-      this.fp.getPrefsService("extensions.foxyproxy.").setBoolPref("firstrun", true);
-    }
-  },
-  
   torWizard : function(firstTime) {
     var owner = foxyproxy._getOptionsDlg();
     if (this.ask(owner, (firstTime ? (this.fp.getMessage("welcome") + " ") : "") +
@@ -186,7 +234,7 @@ var foxyproxy = {
    * Open or focus the main window/dialog
    */
   onOptionsDialog : function() {
-    this.onDialog("foxyproxy-options", "chrome://foxyproxy/content/options.xul", "foxyproxy-quickadd");
+    this.onDialog("foxyproxy-options", "chrome://foxyproxy/content/options.xul", null, null, "foxyproxy-superadd");
   },
 
   onDialog : function(id, xulFile, args, parms, idToClose) {
@@ -211,45 +259,19 @@ var foxyproxy = {
   },
 
   onQuickAddDialog : function(evt) {
-  	var fp=this.fp;
-		if (fp.mode != "disabled" && fp.quickadd.enabled) {
+		if (this.fp.mode != "disabled" && this.fp.quickadd.enabled) {
 		  if (!evt.view || !evt.view.content || !evt.view.content.document || !evt.view.content.document.location) {
 		  	fp.notifier.alert(fp.getMessage("foxyproxy"), fp.getMessage("quickadd.nourl"));
 		  	return;
 		  }
-			if (fp.quickadd._prompt) {
-				// Close Options dialog if it's open to prevent sync problems between the two dialogs
-			  var p = {out:null};
-				this.onDialog("foxyproxy-quickadd", "chrome://foxyproxy/content/quickadd.xul", "modal", p, "foxyproxy-options");
-				if (p.out) {
- 					fp.quickadd.reload = p.out.reload;
- 					fp.quickadd.notify = p.out.notify;
- 					fp.quickadd.prompt = p.out.prompt;
- 					fp.quickadd.notifyWhenCanceled = p.out.notifyWhenCanceled;
- 					foxyproxy_common.onQuickAddProxyChanged(p.out.proxy);
- 					fp.quickadd.urlTemplate = p.out.urlTemplate;
- 					fp.quickadd.setMatchIsRegEx(p.out.matchType=="r");
-                    fp.quickadd.caseSensitive = p.out.caseSensitive;
-					_qAdd(p.out.url);
-				}
-				// if !p.out then the user canceled QuickAdd
-			}
-			else {
-				_qAdd();
-			}
-		}
-		function _qAdd(h) {
-			h = h || evt.view.content.document.location.href;
-			var m = fp.quickadd._proxy.isMatch(h);
-			if (m) {
-		    fp.quickadd._notify &&
-		    	fp.notifier.alert(fp.getMessage("foxyproxy.quickadd.label"), fp.getMessage("quickadd.quickadd.canceled", [m.name, fp.quickadd._proxy.name]));
-			}
-			else {
-				fp.quickadd.addPattern(evt.view.content.document.location, h);
-				fp.writeSettings();
-			}
-		}
+      this.fp.quickadd.onQuickAdd(window, evt.view.content.document);
+    }
+  },
+
+  onPageLoad : function(evt) {
+    var doc = evt.originalTarget; // doc is document that triggered "onload" event
+    if (doc && doc.location)
+      foxyproxy.fp.autoadd.onAutoAdd(window, doc); // can't use |this.fp| because this isn't |foxyproxy|
   },
 
   updateViews : function(writeSettings, updateLogView) {
@@ -303,11 +325,11 @@ var foxyproxy = {
   },
 
   checkPageLoad : function() {
-  	var e = this.fp.mode != "disabled" && this.fp.autoadd.enabled;
+  	var listen = this.fp.mode != "disabled" && this.fp.autoadd.enabled;
     var appcontent = document.getElementById("appcontent");
     if (appcontent) {
 			appcontent.removeEventListener("load", this.onPageLoad, true); // safety
-	    if (e) {
+	    if (listen) {
 	 	    appcontent.addEventListener("load", this.onPageLoad, true);
 		 	}
 			else {
@@ -348,13 +370,13 @@ var foxyproxy = {
   	// otherwise we get a JS error.
     s && (s.hidden = !e);
   },
-  
+
   toggleStatusBarWidth : function(w) {
     var s=document.getElementById("foxyproxy-status-text");
     // Statusbars don't exist on all windows (e.g,. View Source) so check for existence first,
     // otherwise we get a JS error.
     if (!s) return;
-    var w = this.fp.statusbar.width; 
+    var w = this.fp.statusbar.width;
     if (w > 0)
       s.width = w;
     else {
@@ -364,8 +386,8 @@ var foxyproxy = {
       if (!s.hidden) {
         s.hidden = true;
         s.hidden = false;
-      }     
-    }    
+      }
+    }
   },
 
   // Set toolbar, statusbar, and context menu text and icon colors
@@ -458,7 +480,7 @@ var foxyproxy = {
 					gBrowser.reloadAllTabs();
 					break;
 				case "reloadtabsinallbrowsers":
-				  for (var b, e = foxyproxy_common.getEnumerator();
+				  for (var b, e = this.fpc.getEnumerator();
 						   	  e.hasMoreElements();
 						   	  (b = e.getNext().getBrowser()) && b.reloadAllTabs());
 					break;
@@ -691,7 +713,7 @@ var foxyproxy = {
 
 	      _createMenuItem(submenupopup,
 	        this.fp.getMessage("foxyproxy.help.label"),
-	        "foxyproxy_common.openAndReuseOneTabPerURL('http://foxyproxy.mozdev.org/quickstart.html');",
+	        "foxyproxy.fpc.openAndReuseOneTabPerURL('http://foxyproxy.mozdev.org/quickstart.html');",
 	        this.fp.getMessage("foxyproxy.help.accesskey"),
 	        this.fp.getMessage("foxyproxy.help.tooltip"));
 
@@ -721,28 +743,6 @@ var foxyproxy = {
 	    }
     }
 
-    // Open link with proxy...
-    /*var target = gContextMenu.target;
-    // target can be null when opening something like chrome://passwdmaker/chrome/uris.xul directly in the browser.
-    if (target && target.tagName == "A") {
-      menupopup.appendChild(document.createElement("menuseparator"));
-      for (var i=0; i<this.fp.proxies.length; i++) {
-        var p = this.fp.proxies.item(i);
-        var pName = p.name;
-        // Set the submenu based on advancedMenus enabled/disabled
-        //var sbm = this.fp.advancedMenus ? _createMenu(menupopup, pName, pName.substring(0, 1), pName) : menupopup;
-        var curProxy = "foxyproxy.fp.proxies.item(" + i + ").";
-	      _createMenuItem(menupopup,
-	        this.fp.getMessage("openwithproxy.label", [pName]),
-	        "alert('yo baby yo');",
-	        this.fp.getMessage("openwithproxy.accesskey"),
-	        this.fp.getMessage("openwithproxy.tooltip"));
-			}
-
-      // Save a reference to the object being clicked
-      foxyproxy.clickedElement = target;
-    }*/
-
     function _createMenu(menupopup, label, accesskey, tooltip) {
       var submenu = document.createElement("menu");
       submenu.setAttribute("label", label);
@@ -791,7 +791,6 @@ var foxyproxy = {
     }
   },
 
-	// common fcns used in overlay.js and options.js
 	onToggleNoURLs : function(owner) {
 		this.fp.logg.noURLs=!this.fp.logg.noURLs;
 		if (this.fp.logg.noURLs && this.fp.logg.length > 0) {
